@@ -29,6 +29,8 @@
 #include <stdio.h>
 #include <mntent.h>
 #include <sys/types.h>
+#include <sys/acl.h>
+#include <errno.h>
 
 #include <glib/gstdio.h>
 
@@ -62,6 +64,7 @@ typedef struct _UDisksLinuxFilesystemClass   UDisksLinuxFilesystemClass;
 struct _UDisksLinuxFilesystem
 {
   UDisksFilesystemSkeleton parent_instance;
+  GMutex lock;
 };
 
 struct _UDisksLinuxFilesystemClass
@@ -76,9 +79,21 @@ G_DEFINE_TYPE_WITH_CODE (UDisksLinuxFilesystem, udisks_linux_filesystem, UDISKS_
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+static void
+udisks_linux_filesystem_finalize (GObject *object)
+{
+  UDisksLinuxFilesystem *filesystem = UDISKS_LINUX_FILESYSTEM (object);
+
+  g_mutex_clear (&(filesystem->lock));
+
+  if (G_OBJECT_CLASS (udisks_linux_filesystem_parent_class)->finalize != NULL)
+    G_OBJECT_CLASS (udisks_linux_filesystem_parent_class)->finalize (object);
+}
+
 static void
 udisks_linux_filesystem_init (UDisksLinuxFilesystem *filesystem)
 {
+  g_mutex_init (&filesystem->lock);
   g_dbus_interface_skeleton_set_flags (G_DBUS_INTERFACE_SKELETON (filesystem),
                                        G_DBUS_INTERFACE_SKELETON_FLAGS_HANDLE_METHOD_INVOCATIONS_IN_THREAD);
 }
@@ -86,6 +101,10 @@ udisks_linux_filesystem_init (UDisksLinuxFilesystem *filesystem)
 static void
 udisks_linux_filesystem_class_init (UDisksLinuxFilesystemClass *klass)
 {
+  GObjectClass *gobject_class;
+
+  gobject_class = G_OBJECT_CLASS (klass);
+  gobject_class->finalize     = udisks_linux_filesystem_finalize;
 }
 
 /**
@@ -493,8 +512,8 @@ is_mount_option_allowed (const FSMountOptions *fsmo,
 
 static gchar **
 prepend_default_mount_options (const FSMountOptions *fsmo,
-                               uid_t caller_uid,
-                               GVariant *given_options)
+                               uid_t                 caller_uid,
+                               GVariant             *given_options)
 {
   GPtrArray *options;
   gint n;
@@ -505,9 +524,11 @@ prepend_default_mount_options (const FSMountOptions *fsmo,
   options = g_ptr_array_new ();
   if (fsmo != NULL)
     {
-      for (n = 0; fsmo->defaults != NULL && fsmo->defaults[n] != NULL; n++)
+      const gchar *const *defaults = fsmo->defaults;
+
+      for (n = 0; defaults != NULL && defaults[n] != NULL; n++)
         {
-          const gchar *option = fsmo->defaults[n];
+          const gchar *option = defaults[n];
 
           if (strcmp (option, "uid=") == 0)
             {
@@ -653,6 +674,7 @@ calculate_fs_type (UDisksBlock               *block,
 
 /*
  * calculate_mount_options: <internal>
+ * @daemon: A #UDisksDaemon.
  * @block: A #UDisksBlock.
  * @caller_uid: The uid of the caller making the request.
  * @fs_type: The filesystem type to use or %NULL.
@@ -665,7 +687,8 @@ calculate_fs_type (UDisksBlock               *block,
  * Returns: A string with mount options or %NULL if @error is set. Free with g_free().
  */
 static gchar *
-calculate_mount_options (UDisksBlock               *block,
+calculate_mount_options (UDisksDaemon              *daemon,
+                         UDisksBlock               *block,
                          uid_t                      caller_uid,
                          const gchar               *fs_type,
                          GVariant                  *options,
@@ -758,8 +781,45 @@ ensure_utf8 (const gchar *s)
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+static gboolean
+add_acl (const gchar  *path,
+         uid_t         uid,
+         GError      **error)
+{
+  gboolean ret = FALSE;
+  acl_t acl = NULL;
+  acl_entry_t entry;
+  acl_permset_t permset;
+
+  acl = acl_get_file(path, ACL_TYPE_ACCESS);
+  if (acl == NULL ||
+      acl_create_entry (&acl, &entry) == -1 ||
+      acl_set_tag_type (entry, ACL_USER) == -1 ||
+      acl_set_qualifier (entry, &uid) == -1 ||
+      acl_get_permset (entry, &permset) == -1 ||
+      acl_add_perm (permset, ACL_READ|ACL_EXECUTE) == -1 ||
+      acl_calc_mask (&acl) == -1 ||
+      acl_set_file (path, ACL_TYPE_ACCESS, acl) == -1)
+    {
+      g_set_error (error,
+                   G_IO_ERROR,
+                   g_io_error_from_errno (errno),
+                   "Adding read ACL for uid %d to `%s' failed: %m",
+                   (gint) uid, path);
+      goto out;
+    }
+
+  ret = TRUE;
+
+ out:
+  if (acl != NULL)
+    acl_free (acl);
+  return ret;
+}
+
 /*
  * calculate_mount_point: <internal>
+ * @dameon: A #UDisksDaemon.
  * @block: A #UDisksBlock.
  * @uid: user id of the calling user
  * @gid: group id of the calling user
@@ -772,7 +832,8 @@ ensure_utf8 (const gchar *s)
  * Returns: A UTF-8 string with the mount point to use or %NULL if @error is set. Free with g_free().
  */
 static gchar *
-calculate_mount_point (UDisksBlock               *block,
+calculate_mount_point (UDisksDaemon              *daemon,
+                       UDisksBlock               *block,
                        uid_t                      uid,
                        gid_t                      gid,
                        const gchar               *user_name,
@@ -797,15 +858,14 @@ calculate_mount_point (UDisksBlock               *block,
       uuid = udisks_block_get_id_uuid (block);
     }
 
-  /* If we know the user-name and it doesn't have any '/' character in it, mount in /run/media/$USER */
+  /* If we know the user-name and it doesn't have any '/' character in
+   * it, mount in /run/media/$USER
+   */
   if (user_name != NULL && strstr (user_name, "/") == NULL)
     {
       mount_dir = g_strdup_printf ("/run/media/%s", user_name);
       if (!g_file_test (mount_dir, G_FILE_TEST_EXISTS))
         {
-          gchar *stderr_txt;
-          gint exit_status;
-
           /* First ensure that /run/media exists */
           if (!g_file_test ("/run/media", G_FILE_TEST_EXISTS))
             {
@@ -828,36 +888,13 @@ calculate_mount_point (UDisksBlock               *block,
                            mount_dir);
               goto out;
             }
-          /* Then set the ACL such that only $USER can actually access it */
-          escaped_user_name = udisks_daemon_util_escape (user_name);;
-          s = g_strdup_printf ("setfacl -m \"u:%s:rx\" \"%s\"",
-                               escaped_user_name,
-                               mount_dir);
-          if (!g_spawn_command_line_sync (s,
-                                          NULL, /* stdout_txt */
-                                          &stderr_txt,
-                                          &exit_status,
-                                          error))
+          /* Finally, add the read+execute ACL for $USER */
+          if (!add_acl (mount_dir, uid, error))
             {
-              g_free (s);
               if (rmdir (mount_dir) != 0)
                 udisks_warning ("Error calling rmdir() on %s: %m", mount_dir);
               goto out;
             }
-          if (!(WIFEXITED (exit_status) && WEXITSTATUS (exit_status) == 0))
-            {
-              g_set_error (error,
-                           UDISKS_ERROR,
-                           UDISKS_ERROR_FAILED,
-                           "Command-line `%s' didn't exit normally: %s", s, stderr_txt);
-              g_free (stderr_txt);
-              g_free (s);
-              if (rmdir (mount_dir) != 0)
-                udisks_warning ("Error calling rmdir() on %s: %m", mount_dir);
-              goto out;
-            }
-          g_free (stderr_txt);
-          g_free (s);
         }
     }
   /* otherwise fall back to mounting in /media */
@@ -1066,6 +1103,7 @@ handle_mount (UDisksFilesystem       *filesystem,
   UDisksBlock *block;
   UDisksDaemon *daemon;
   UDisksCleanup *cleanup;
+  pid_t caller_pid;
   uid_t caller_uid;
   gid_t caller_gid;
   const gchar * const *existing_mount_points;
@@ -1081,6 +1119,7 @@ handle_mount (UDisksFilesystem       *filesystem,
   gchar *caller_user_name;
   GError *error;
   const gchar *action_id;
+  const gchar *message;
   gboolean system_managed;
   gchar *escaped_device = NULL;
 
@@ -1096,6 +1135,9 @@ handle_mount (UDisksFilesystem       *filesystem,
   caller_user_name = NULL;
   system_managed = FALSE;
 
+  /* only allow a single call at a time */
+  g_mutex_lock (&UDISKS_LINUX_FILESYSTEM (filesystem)->lock);
+
   error = NULL;
   object = udisks_daemon_util_dup_object (filesystem, &error);
   if (object == NULL)
@@ -1151,6 +1193,18 @@ handle_mount (UDisksFilesystem       *filesystem,
       goto out;
     }
 
+  error = NULL;
+  if (!udisks_daemon_util_get_caller_pid_sync (daemon,
+                                               invocation,
+                                               NULL /* GCancellable */,
+                                               &caller_pid,
+                                               &error))
+    {
+      g_dbus_method_invocation_return_gerror (invocation, error);
+      g_error_free (error);
+      goto out;
+    }
+
   if (system_managed)
     {
       gint status;
@@ -1159,14 +1213,30 @@ handle_mount (UDisksFilesystem       *filesystem,
       if (!has_option (fstab_mount_options, "x-udisks-auth"))
         {
           action_id = "org.freedesktop.udisks2.filesystem-mount";
-          if (udisks_block_get_hint_system (block) &&
-              !(udisks_daemon_util_setup_by_user (daemon, object, caller_uid)))
-            action_id = "org.freedesktop.udisks2.filesystem-mount-system";
+          /* Translators: Shown in authentication dialog when the user
+           * requests mounting a filesystem.
+           *
+           * Do not translate $(udisks2.device), it's a placeholder and
+           * will be replaced by the name of the drive/device in question
+           */
+          message = N_("Authentication is required to mount $(udisks2.device)");
+          if (!udisks_daemon_util_setup_by_user (daemon, object, caller_uid))
+            {
+              if (udisks_block_get_hint_system (block))
+                {
+                  action_id = "org.freedesktop.udisks2.filesystem-mount-system";
+                }
+              else if (!udisks_daemon_util_on_same_seat (daemon, object, caller_pid))
+                {
+                  action_id = "org.freedesktop.udisks2.filesystem-mount-other-seat";
+                }
+            }
+
           if (!udisks_daemon_util_check_authorization_sync (daemon,
                                                             object,
                                                             action_id,
                                                             options,
-                                                            N_("Authentication is required to mount $(udisks2.device)"),
+                                                            message,
                                                             invocation))
             goto out;
           mount_fstab_as_root = TRUE;
@@ -1207,7 +1277,17 @@ handle_mount (UDisksFilesystem       *filesystem,
                                                                 object,
                                                                 "org.freedesktop.udisks2.filesystem-fstab",
                                                                 options,
-                                                                N_("Authentication is required to mount the fstab device $(udisks2.device)"),
+                                                                /* Translators: Shown in authentication dialog when the
+                                                                 * user requests mounting a filesystem that is in
+                                                                 * /etc/fstab file with the x-udisks-auth option.
+                                                                 *
+                                                                 * Do not translate $(udisks2.device), it's a
+                                                                 * placeholder and will be replaced by the name of
+                                                                 * the drive/device in question
+                                                                 *
+                                                                 * Do not translate /etc/fstab
+                                                                 */
+                                                                N_("Authentication is required to mount $(udisks2.device) referenced in the /etc/fstab file"),
                                                                 invocation))
                 goto out;
               mount_fstab_as_root = TRUE;
@@ -1276,7 +1356,8 @@ handle_mount (UDisksFilesystem       *filesystem,
 
   /* calculate mount options (guaranteed to be valid UTF-8) */
   error = NULL;
-  mount_options_to_use = calculate_mount_options (block,
+  mount_options_to_use = calculate_mount_options (daemon,
+                                                  block,
                                                   caller_uid,
                                                   fs_type_to_use,
                                                   options,
@@ -1293,20 +1374,37 @@ handle_mount (UDisksFilesystem       *filesystem,
    * may be racing with other threads...
    */
   action_id = "org.freedesktop.udisks2.filesystem-mount";
-  if (udisks_block_get_hint_system (block) &&
-      !(udisks_daemon_util_setup_by_user (daemon, object, caller_uid)))
-    action_id = "org.freedesktop.udisks2.filesystem-mount-system";
+  /* Translators: Shown in authentication dialog when the user
+   * requests mounting a filesystem.
+   *
+   * Do not translate $(udisks2.device), it's a placeholder and
+   * will be replaced by the name of the drive/device in question
+   */
+  message = N_("Authentication is required to mount $(udisks2.device)");
+  if (!udisks_daemon_util_setup_by_user (daemon, object, caller_uid))
+    {
+      if (udisks_block_get_hint_system (block))
+        {
+          action_id = "org.freedesktop.udisks2.filesystem-mount-system";
+        }
+      else if (!udisks_daemon_util_on_same_seat (daemon, object, caller_pid))
+        {
+          action_id = "org.freedesktop.udisks2.filesystem-mount-other-seat";
+        }
+    }
+
   if (!udisks_daemon_util_check_authorization_sync (daemon,
                                                     object,
                                                     action_id,
                                                     options,
-                                                    N_("Authentication is required to mount $(udisks2.device)"),
+                                                    message,
                                                     invocation))
     goto out;
 
   /* calculate mount point (guaranteed to be valid UTF-8) */
   error = NULL;
-  mount_point_to_use = calculate_mount_point (block,
+  mount_point_to_use = calculate_mount_point (daemon,
+                                              block,
                                               caller_uid,
                                               caller_gid,
                                               caller_user_name,
@@ -1390,6 +1488,9 @@ handle_mount (UDisksFilesystem       *filesystem,
   g_free (caller_user_name);
   g_clear_object (&object);
 
+  /* only allow a single call at a time */
+  g_mutex_unlock (&UDISKS_LINUX_FILESYSTEM (filesystem)->lock);
+
   return TRUE; /* returning TRUE means that we handled the method invocation */
 }
 
@@ -1436,6 +1537,9 @@ handle_unmount (UDisksFilesystem       *filesystem,
   error_message = NULL;
   opt_force = FALSE;
 
+  /* only allow a single call at a time */
+  g_mutex_lock (&UDISKS_LINUX_FILESYSTEM (filesystem)->lock);
+
   error = NULL;
   object = udisks_daemon_util_dup_object (filesystem, &error);
   if (object == NULL)
@@ -1517,7 +1621,17 @@ handle_unmount (UDisksFilesystem       *filesystem,
                                                                 object,
                                                                 "org.freedesktop.udisks2.filesystem-fstab",
                                                                 options,
-                                                                N_("Authentication is required to unmount the fstab device $(udisks2.device)"),
+                                                                /* Translators: Shown in authentication dialog when the
+                                                                 * user requests unmounting a filesystem that is in
+                                                                 * /etc/fstab file with the x-udisks-auth option.
+                                                                 *
+                                                                 * Do not translate $(udisks2.device), it's a
+                                                                 * placeholder and will be replaced by the name of
+                                                                 * the drive/device in question
+                                                                 *
+                                                                 * Do not translate /etc/fstab
+                                                                 */
+                                                                N_("Authentication is required to unmount $(udisks2.device) referenced in the /etc/fstab file"),
                                                                 invocation))
                 goto out;
               unmount_fstab_as_root = TRUE;
@@ -1553,11 +1667,24 @@ handle_unmount (UDisksFilesystem       *filesystem,
 
   if (caller_uid != 0 && (caller_uid != mounted_by_uid))
     {
+      const gchar *action_id;
+      const gchar *message;
+
+      action_id = "org.freedesktop.udisks2.filesystem-unmount-others";
+      /* Translators: Shown in authentication dialog when the user
+       * requests unmounting a filesystem previously mounted by
+       * another user.
+       *
+       * Do not translate $(udisks2.device), it's a placeholder and
+       * will be replaced by the name of the drive/device in question
+       */
+      message = N_("Authentication is required to unmount $(udisks2.device) mounted by another user");
+
       if (!udisks_daemon_util_check_authorization_sync (daemon,
                                                         object,
-                                                        "org.freedesktop.udisks2.filesystem-unmount-others",
+                                                        action_id,
                                                         options,
-                                                        N_("Authentication is required to unmount $(udisks2.device) mounted by another user"),
+                                                        message,
                                                         invocation))
         goto out;
     }
@@ -1622,6 +1749,9 @@ handle_unmount (UDisksFilesystem       *filesystem,
   g_free (mount_point);
   g_free (fstab_mount_options);
   g_clear_object (&object);
+
+  g_mutex_unlock (&UDISKS_LINUX_FILESYSTEM (filesystem)->lock);
+
   return TRUE;
 }
 
@@ -1663,6 +1793,8 @@ handle_set_label (UDisksFilesystem       *filesystem,
   const FSInfo *fs_info;
   UDisksBaseJob *job;
   const gchar *action_id;
+  const gchar *message;
+  pid_t caller_pid;
   gchar *command;
   gchar *tmp;
   GError *error;
@@ -1682,6 +1814,18 @@ handle_set_label (UDisksFilesystem       *filesystem,
   daemon = udisks_linux_block_object_get_daemon (UDISKS_LINUX_BLOCK_OBJECT (object));
   block = udisks_object_peek_block (object);
 
+  error = NULL;
+  if (!udisks_daemon_util_get_caller_pid_sync (daemon,
+                                               invocation,
+                                               NULL /* GCancellable */,
+                                               &caller_pid,
+                                               &error))
+    {
+      g_dbus_method_invocation_return_gerror (invocation, error);
+      g_error_free (error);
+      goto out;
+    }
+
   probed_fs_usage = udisks_block_get_id_usage (block);
   probed_fs_type = udisks_block_get_id_type (block);
 
@@ -1746,8 +1890,21 @@ handle_set_label (UDisksFilesystem       *filesystem,
     }
 
   action_id = "org.freedesktop.udisks2.modify-device";
+  /* Translators: Shown in authentication dialog when the user
+   * requests changing the filesystem label.
+   *
+   * Do not translate $(udisks2.device), it's a placeholder and
+   * will be replaced by the name of the drive/device in question
+   */
+  message = N_("Authentication is required to change the filesystem label on $(udisks2.device)");
   if (udisks_block_get_hint_system (block))
-    action_id = "org.freedesktop.udisks2.modify-device-system";
+    {
+      action_id = "org.freedesktop.udisks2.modify-device-system";
+    }
+  else if (!udisks_daemon_util_on_same_seat (daemon, UDISKS_OBJECT (object), caller_pid))
+    {
+      action_id = "org.freedesktop.udisks2.modify-device-other-seat";
+    }
 
   /* Check that the user is actually authorized to change the
    * filesystem label.
@@ -1756,7 +1913,7 @@ handle_set_label (UDisksFilesystem       *filesystem,
                                                     object,
                                                     action_id,
                                                     options,
-                                                    N_("Authentication is required to change the filesystem label on $(udisks2.device)"),
+                                                    message,
                                                     invocation))
     goto out;
 
